#include <string>
#include "menu.h"
#include "system.h"

using namespace std;

namespace hm
{

	Menu::Menu(string title,Surface *surface) :
		selected(0),
		icon(NULL),
		secondaryIcon(NULL),
		scroll(0),
		actionHandler(NULL),
		background(NULL),
		parent(NULL),
		x(-1),
		y(-1)

	{
		this->title = title;
		this->surface = surface;
		setWindowSize(220,160);
	}

	Menu::~Menu()
	{
		// delete all children
		for (int i=0; i<children.size(); i++)
			delete children[i];
		children.clear();
	}

	void Menu::draw()
	{
		Surface *surface = getSurface();

		surface->clearClipRect();

		// draw background
		if (background!=NULL)
			surface->drawImage(background,0,0);

		// center the window to the surface
		if ((x==-1) && (y==-1))
		{
			x = (surface->getWidth() - width) / 2;
			y = (surface->getHeight() - height) / 2;
		}

		// draw the main window
		surface->drawWindow(x,y,width,height,title);

		// draw the item list
		int listW = width - windowPadding*2;
		int listH = height - windowPadding*2 - surface->getHeaderHeight();
		int listX = x + windowPadding;
		int listY = y + windowPadding + surface->getHeaderHeight();
		drawList(listX,listY,listW,listH);

		surface->clearClipRect();
		surface->flip();
	}

	void Menu::drawList(int x,int y,int w,int h)
	{

		Surface *surface = getSurface();

		// set clipping rect
		surface->setClipRect(x,y,w,h);

		// calculate total height of all items
		int height = 0;
		for (unsigned int i=0; i<children.size(); i++)
			height += children.at(i)->getListItemHeight();

		bool drawScrollBar = false;
		if (height>h)
		{
			w -= surface->getScrollBarWidth();
			drawScrollBar = true;
		}

		if (surface->getSkinColor(LISTBACKGROUND)!=0xff00ff)
			surface->drawRect(x,y,w,h,surface->getSkinColor(LISTBACKGROUND));

		// check if our selected item is within the listbox area
		int selectedItemPos = 0;
		int selectedItemHeight = 0;
		if (children.size()>selected)
		{
			selectedItemHeight = children.at(selected)->getListItemHeight();
			for (int i=0; i<selected; i++)
				selectedItemPos += children.at(i)->getListItemHeight();
		}
		selectedItemPos -= scroll;

		int selectedItemBottom = selectedItemPos+selectedItemHeight;

		if (selectedItemBottom>h) scroll += selectedItemBottom - h;
		if (selectedItemPos<0) scroll -= -selectedItemPos;

		// draw items
		int ypos = -scroll;
		for (unsigned int i=0; ((i<children.size())&&(ypos<h)); i++)
		{

			int itemHeight = children.at(i)->getListItemHeight();
			int itemPos = ypos;

			if (itemPos+itemHeight>0)
			{

				// clip against the list window
				int clipY = ypos;
				int clipHeight = itemHeight;

				if (clipY<0)
				{
					clipY = 0;
					clipHeight -= -ypos;
				}

				if (clipY+clipHeight>h)
					clipHeight -= (clipY+clipHeight) - h;

				surface->setClipRect(x,y+clipY,w,clipHeight);

				if (selected==i)
				{
					surface->drawRect(x+windowPadding,y+itemPos,w-(windowPadding*2),itemHeight,surface->getSkinColor(LISTSELECTED));
				}

				children.at(i)->drawListItem(surface,x+windowPadding,y+ypos,w-(windowPadding*2),itemHeight,selected==i);
			}

			ypos += itemHeight;
		}

		// draw scrollbar
		if (drawScrollBar)
			surface->drawScrollBar(x+w,y,h,scroll,height-h);

		// un-set clipping rect
		surface->clearClipRect();

	}

	void Menu::drawListItem(Surface *surface,int x,int y,int w,int h,bool selected)
	{

		int temp_x=x;
		int temp_y=y;

		temp_x+=listItemPadding;
		temp_y+=listItemPadding;

		if (icon!=NULL)
		{
			surface->drawImage(icon,temp_x,temp_y);
			temp_x += icon->getWidth() + listItemPadding;
			temp_y += (icon->getHeight()-surface->getFont(selected?SELECTED:NORMAL)->getHeight()) / 2;
		}
		surface->drawText(title,temp_x,temp_y,selected?SELECTED:NORMAL);

		/*
		   if (children.size()>0) 
		   surface->drawImage(
		   surface->getSkinElement(SUBMENU_ICON),
		   x+w-surface->getSkinElement(SUBMENU_ICON)->getWidth()-4,
		   temp_y);
		   */
	}

	int Menu::getListItemHeight()
	{

		Surface *surface = getSurface();

		int h = surface->getFont(NORMAL)->getHeight();
		if ((icon!=NULL)&&(icon->getHeight()>h)) h = icon->getHeight();
		return( h + listItemPadding*2 );
	}

	void Menu::setWindowSize(int w,int h)
	{
		width = w;
		height = h;
	}

	void Menu::setWindowPosition(int x,int y)
	{
		this->x = x;
		this->y = y;
	}

	// takes ownership of the child
	void Menu::appendChild(Menu* menu)
	{
		menu->parent = this;
		children.push_back(menu);
	}

	int Menu::enter()
	{

		int res = 0;

		// fire enter event
		if (actionHandler!=NULL)
			actionHandler->enter(this);

		// init
		Surface *surface = getSurface();
		scroll = 0;
		running=true;

		// backup surface contents
		background = surface->toImage();

		// draw window
		draw();

		while (running)
		{
			Event e = msystem.pollEvent();
			if (e.type==EVENT_QUIT) running = false;

			if (e.type==EVENT_KEYUP)
			{
				if (e.key==KEY_REJECT)
					running=false;

				if (e.key==KEY_ACCEPT)
				{
					if (children.size()>selected)
					{
						res = children.at(selected)->enter();
						if (res)
							break;
					}
					draw();
				}
			}

			if (e.type==EVENT_KEYDOWN)
			{
				if ((e.key==KEY_DOWN)&&(selected<children.size()-1))
				{
					int temp = selected + 1;
					while ((!children.at(temp)->isSelectable()) && (temp<children.size()))
						temp ++;
					if (children.at(temp)->isSelectable())
						selected = temp;
				}

				if ((e.key==KEY_UP)&&(selected>0))
				{
					int temp = selected - 1;
					while ((!children.at(temp)->isSelectable()) && (temp))
						temp --;
					if (children.at(temp)->isSelectable())
						selected = temp;
				}

				draw();
			}

			if (e.type == EVENT_TIMER)
			{
				static int i = 0;

				// every 10 seconds, force a display update
				i++;
				if (i > 333)
				{
					i = 0;
					draw();
				}
			}
		}

		delete(background);
		background = NULL;

		return res;
	}

	void Menu::close()
	{

		running = false;

		for (int i=0; i<children.size(); i++)
			children.at(i)->close();
	}


	void Menu::setIcon(Image *icon)
	{
		this->icon = icon;        
	}

	Image* Menu::getIcon()
	{
		return(icon);
	}

	void Menu::setSecondaryIcon(Image *icon)
	{
		secondaryIcon = icon;        
	}

	Image* Menu::getSecondaryIcon()
	{
		return(secondaryIcon);
	}

	string Menu::getTitle()
	{
		return(title);
	}

	void Menu::setTitle(string text)
	{
		title = text;
	}


	void Menu::setName(string name)
	{
		this->name = name;
	}

	void Menu::setValue(string value)
	{
		this->value = value;
	}

	string Menu::getName()
	{
		return(name);
	}

	string Menu::getValue()
	{
		return(value);
	}

	void Menu::setActionHandler(ActionHandler *handler)
	{
		actionHandler = handler;
	}

	void Menu::addValues(ConfigFile *config)
	{

		// add the name,value of this node
		if ((name!="")&&(value!=""))
			config->addVariable(name,value);

		// add all children
		for (int i=0; i<children.size(); i++)
			children.at(i)->addValues(config);
	}

	void Menu::getValues(ConfigFile *config)
	{

		// check if our value is in the file
		if (name!="")
		{
			string val = config->getValueByName(name);
			if (val!="")
			{
				value = val;
				// tell our event handler our state has changed
				if (actionHandler!=NULL)
					actionHandler->change(this);
			}
		}

		// add all children
		for (int i=0; i<children.size(); i++)
			children.at(i)->getValues(config);
	}

	void Menu::setSurface(Surface *surf)
	{
		surface = surf;
	}

	Menu* Menu::getParent()
	{
		return(parent);
	}

	Surface* Menu::getSurface()
	{
		if (surface!=NULL)
			return(surface); else
				return(parent->getSurface());
	}

	void Menu::setSelected (int selected)
	{
		if ( (selected<children.size()) &&
				(selected>=0) )
			this->selected = selected;
	}

	bool Menu::isSelectable (void)
	{
		return true;
	}
};

