#ifndef __surface_h__
#define __surface_h__

#include "image.h"
#include "font.h"

namespace hm {

	typedef enum {
		NORMAL = 0,
		SELECTED = 1,
		HEADER = 2,
		NR_FONTS = 3
	} FONT_TYPE;
		
	typedef enum {
		LISTBACKGROUND = 0,
		LISTSELECTED = 1,
		NUM_SKIN_COLORS = 2
	} SKIN_COLOR;

	typedef enum {
		HEADER_LEFT = 0,
		HEADER_CENTER = 1,
		HEADER_RIGHT = 2,
		WINDOW_BORDER_LEFT = 3,
		WINDOW_BORDER_RIGHT = 4,
		WINDOW_BORDER_BOTTOM_LEFT = 5,
		WINDOW_BORDER_BOTTOM_RIGHT = 6,
		WINDOW_BORDER_BOTTOM = 7,
		WINDOW_FILL = 8,

		BUTTON = 9,
		BUTTON_DOWN = 10,
		BUTTON_SELECTED = 11,
		BUTTON_SELECTED_DOWN = 12,

		SCROLLBAR_UP = 13,
		SCROLLBAR_BACK = 14,
		SCROLLBAR_DOWN = 15,
		SCROLLBAR_BUTTON_TOP = 16,
		SCROLLBAR_BUTTON_CENTER = 17,
		SCROLLBAR_BUTTON_BOTTOM = 18,

		CHECKBOX = 19,
		CHECKBOX_CHECKED = 20,

		ALERT_ICON = 21,
		SUBMENU_ICON = 22,

		NUM_HEADER_FILES = 23
	} SKIN_ELEMENT;

	/**
	*
	*	The Surface class takes care of all the graphics rendering. You can use these functions when implementing
	*	your own Menu objects.
	*
	*/
	class Surface {

	public:
		
		/** constructor
			\param w width
			\param h height
			*/
		Surface(int w,int h);

		/** destructor */
		~Surface();

        /** after you've done all your drawing, you should call this function to actually put the things on screen */
		void flip();

		/** if you want to draw pixels directly to the backbuffer, you can use this to get a pointer to the current
			backbuffer. Format is unsigned chars, 24bpp */
		unsigned char* getBuffer();

		/** \returns <int> the width of the surface */
		int getWidth();

		/** \returns <int> the height of the surface */
		int getHeight();

		/** draws an image object to the surface 
			\param image Image object to draw
			\param x horizontal position
			\param y vertical position
			*/
		void drawImage(Image *image,int x,int y);

		/** draws a window, with a header and a title
			\param x horizontal position
			\param y vertical position
			\param w width
			\param h height
			\param title text displayed in the window header
			*/
		void drawWindow(int x,int y,int w,int h,string title);

		/** draws a string of text to the surface. 
			\param text the text to draw
			\param x horizontal position
			\param y vertical position
			\param type either NORMAL, SELECTED or HEADER, may be omitted (default is NORMAL)			
			*/
		void drawText(string text,int x,int y,FONT_TYPE type = NORMAL);

		/** draws a button
			\param text text on the button face
			\param x horizontal position
			\param y vertical position
			\param down wether or not the button is pressed
			\param selected indicates the button is selected
			*/
		void drawButton(string text,int x,int y,bool down,bool selected);

		/** draws a filled rectangle
			\param x horizontal position
			\param y vertical position
			\param w width
			\param h height
			\param color color to fill the rectangle with (24bpp)
			*/
		void drawRect(int x,int y,int w,int h,int color);

		/** sets the clipping rectangle
			\param x horizontal position
			\param y vertical position
			\param w width
			\param h height
			*/
		void setClipRect(int x,int y,int w,int h);

		/** 'clears' the clipping rectangle - basically sets the clipping rectangle to fit the entire
			surface.
			*/
		void clearClipRect();

		/** draws a scrollbar. The width is determined by the size of the skin images and can be requested by calling
			getScrollBarWidth. 
			The value and range parameters are used to determine the position and height of the scrollbar button. 
			\param x horizontal position
			\param y vertical position
			\param h height
			\param value the current value of the scrollbar
			\param range of possible values
			*/
		void drawScrollBar(int x,int y,int h,int value,int range);

		/** draws a checkbox. Dimensions are determined by the skin files.
			\param x horizontal position
			\param y vertical position
			\param checked check toggle
			*/
		void drawCheckbox(int x,int y,bool checked);

		/** \returns <int> the height of the window header, determined by the skin files
			*/
		int getHeaderHeight();

		/** \returns <int> the width of the scrollbar, determined by the skin files
			*/
		int getScrollBarWidth();

		/** \returns <int> the size of the checkbox (assuming the checkbox images are square and the same size)
			*/
		int getCheckboxSize();

		/** \returns <int> the height of a button. The system assumed all button images are the same size.
			*/
		int getButtonHeight();

		/** \returns <int> the width of a button. The system assumed all button images are the same size.
			*/
		int getButtonWidth();

		/** Draws the alert icon (used for the confirmation popup window)
			*/
		void Surface::drawAlertIcon(int x,int y);

		/** \returns <Font*> the Font object associated with the given type
			\param type either NORMAL, SELECTED or HEADER, may be omitted (default is NORMAL)			
			*/
		Font* getFont(FONT_TYPE type = NORMAL);

		/** \returns <Image*> an Image representation of this surface.
			*/
		Image* toImage();

		/** \returns <Image*> a part of the skin
			*/
		Image* getSkinElement(SKIN_ELEMENT skinElem);

		/** \returns <int> a skin related color defined in skin.cfg
			*/
		int getSkinColor(SKIN_COLOR color);

		/** sets a skin color on the fly
			@param color the color you want to set
			*/
		void setSkinColor(SKIN_COLOR color,int c);


	private:

		void init();

		// skin elements
		Image* skinImages[NUM_HEADER_FILES];

		// skin colors
		int skinColors[NUM_SKIN_COLORS];

		int width,height;
		int *pixels;

		int clipX,clipY,clipW,clipH;

		int numBuffers,curBuffer,lastBuffer;
		unsigned char **buffers;

		// Font
		Font *fonts[NR_FONTS];
		
	};

};

#endif
